//############################################################################

//  Author:	Sharjith Nair
//  Email:	sharjith_ssn@hotmail.com

// Program:	OpenGL, C++ MFC StereoLithography Data File Viewer Application

//							Important Notice:

//	This Idea and the Application is Copyright(c) Sharjith Nair 2002, 2003.
//	You can freely use it as long as you credit me for it.

//	No guarantee/warranty is given on this app and I will not be responsible 
//	for any damage to you, your property or any other person from using it.
//	USE IT ON YOUR OWN RISK.

//	Thankyou
//	Sharjith Nair.

//#############################################################################

// AxisSystem.h: interface for the CAxisSystem class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AXISSYSTEM_H__9C6F4BC8_FF32_413B_AEB1_D38B5962F89E__INCLUDED_)
#define AFX_AXISSYSTEM_H__9C6F4BC8_FF32_413B_AEB1_D38B5962F89E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Geometry.h"
#include "Point3D.h"
#include "Vector3D.h"

class CPoint3D;
class CVector3D;

class CAxisSystem : public CGeometry  
{
public:
	CAxisSystem();
	CAxisSystem(const CPoint3D& Pos, const CVector3D& Dir, const CVector3D& XDir);
	CAxisSystem(const CPoint3D& Pos, const CVector3D& Dir);
	CAxisSystem(const CAxisSystem&);
	~CAxisSystem();
	void SetPosition(const CPoint3D& P);
	void SetPosition(double x, double y, double z);
	void SetDirection(const CVector3D& D);
	void SetDirection(double dx, double dy, double dz);
	void SetXDirection(const CVector3D& XD);
	void SetXDirection(double dxx, double dxy, double dxz);
	void SetYDirection(const CVector3D& YD);
	void SetYDirection(double dyx, double dyy, double dyz);
	CPoint3D GetPosition() const;
	CVector3D GetOrigin() const;
	CVector3D GetDirection() const;
	CVector3D GetXDirection() const;
	CVector3D GetYDirection() const;
	bool operator == (const CAxisSystem& C) const;
	virtual void Translate(const COneAxis&, const double&);
	virtual void Translate(double dx, double dy, double dz);
	virtual void Translate(const CVector3D&);
	virtual void Translate(const CPoint3D& P1, const CPoint3D& P2);
	virtual void Rotate(const COneAxis&, double);
	virtual void Scale(const CPoint3D&, double);
	virtual void Mirror(const CPoint3D&);
	virtual void Mirror(const COneAxis&);
	virtual void Mirror(const CPlane&);
private:
	CPoint3D Position;
	CVector3D Origin;
	CVector3D Direction;
	CVector3D XDirection;
	CVector3D YDirection;
};

#endif // !defined(AFX_AXISSYSTEM_H__9C6F4BC8_FF32_413B_AEB1_D38B5962F89E__INCLUDED_)
