//############################################################################

//  Author:	Sharjith Nair
//  Email:	sharjith_ssn@hotmail.com

// Program:	OpenGL, C++ MFC StereoLithography Data File Viewer Application

//							Important Notice:

//	This Idea and the Application is Copyright(c) Sharjith Nair 2002, 2003.
//	You can freely use it as long as you credit me for it.

//	No guarantee/warranty is given on this app and I will not be responsible 
//	for any damage to you, your property or any other person from using it.
//	USE IT ON YOUR OWN RISK.

//	Thankyou
//	Sharjith Nair.

//#############################################################################

// Curve.h: interface for the CCurve class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CURVE_H__14D97D93_F8D7_4434_AC43_2056A73CF2C2__INCLUDED_)
#define AFX_CURVE_H__14D97D93_F8D7_4434_AC43_2056A73CF2C2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Geometry.h"

class CPoint3D;

enum CurveType { BEZIER, BSPLINE, CIRCLE, ELLIPSE, HYPERBOLA, LINE, PARABOLA };

//Abstract Base Class
class CCurve : public CGeometry  
{
public:
	CCurve();
	virtual ~CCurve();
	virtual void Reverse() = 0;
	virtual CPoint3D PointAtPara(const double& para) = 0;
	virtual double FirstParameter() const = 0;
	virtual double LastParameter() const = 0;
	virtual bool IsClosed() const = 0;
	CurveType Type() const;
	bool IsOfType(const CurveType&) const;
	virtual CCurve* Copy() const = 0;
	virtual void Translate(double dx, double dy, double dz) = 0;
	virtual void Translate(const CVector3D&) = 0;
	virtual void Translate(const CPoint3D&, const CPoint3D&) = 0;
	virtual void Rotate(const COneAxis&, double) = 0;
	virtual void Scale(const CPoint3D&, double) = 0;
	virtual void Mirror(const CPoint3D&) = 0;
	virtual void Mirror(const COneAxis&) = 0;
	virtual void Mirror(const CPlane&) = 0;
protected:
	CurveType cType;
	bool cReversed;
};

#endif // !defined(AFX_CURVE_H__14D97D93_F8D7_4434_AC43_2056A73CF2C2__INCLUDED_)
