//############################################################################

//  Author:	Sharjith Nair
//  Email:	sharjith_ssn@hotmail.com

// Program:	OpenGL, C++ MFC StereoLithography Data File Viewer Application

//							Important Notice:

//	This Idea and the Application is Copyright(c) Sharjith Nair 2002, 2003.
//	You can freely use it as long as you credit me for it.

//	No guarantee/warranty is given on this app and I will not be responsible 
//	for any damage to you, your property or any other person from using it.
//	USE IT ON YOUR OWN RISK.

//	Thankyou
//	Sharjith Nair.

//#############################################################################

// Plane.h: interface for the CPlane class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PLANE_H__10484064_B0F8_46EC_8C2E_DDC8A0EA6299__INCLUDED_)
#define AFX_PLANE_H__10484064_B0F8_46EC_8C2E_DDC8A0EA6299__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Geometry.h"
#include "Point3D.h"
#include "Vector3D.h"
#include "OneAxis.h"
#include "AxisSystem.h"

class CPoint3D;
class CVector3D;
class COneAxis;
class CAxisSystem;

class CPlane : public CGeometry
{
public:
	CPlane(){}
	CPlane(const CPoint3D&, const CPoint3D&, const CPoint3D&);
	CPlane(const CPoint3D&, const CVector3D&, const CVector3D&);
	CPlane(const double&, const double&, const double&, const double&);
	CPlane(const COneAxis&);
	virtual ~CPlane();
	virtual CPoint3D PointAtPara(const double uPar, const double vPar);
	CPoint3D GetPosition() const;
	CVector3D GetDirection() const;
	void EquationAt(const CPoint3D&, double*);
	double GetConstant() const;
	virtual void Translate(const COneAxis&, const double&);
	virtual void Translate(double dx, double dy, double dz);
	virtual void Translate(const CVector3D&);
	virtual void Translate(const CPoint3D&, const CPoint3D&);
	virtual void Rotate(const COneAxis&, double);
	virtual void Scale(const CPoint3D&, double);
	virtual void Mirror(const CPoint3D&);
	virtual void Mirror(const COneAxis&);
	virtual void Mirror(const CPlane&);
	static CPlane XOY();
	static CPlane YOZ();
	static CPlane ZOX();
private:
	void UpdateEquation();
private:
	double Equation[4];	//Plane Equation
	CAxisSystem itsLocation;

};

#endif // !defined(AFX_PLANE_H__10484064_B0F8_46EC_8C2E_DDC8A0EA6299__INCLUDED_)
